@@ -263,6 +263,7 @@ esc_to_char($t) -> $\t.
 
 scan_number([]) -> {more, []};
 scan_number(eof) -> {done, {error, incomplete_number}, []};
+scan_number([$-, $- | _Ds]) -> {done, {error, invalid_number}, []};
 scan_number([$- | Ds] = Input) ->
     case scan_number(Ds) of
 	{more, _Cont} -> {more, Input};
@@ -475,7 +476,7 @@ parse_object2(Obj, S, C, Kv) ->
 parse_array(C, Kv) ->
     get_token(C, fun
         (eof, C2) -> {done, {error, premature_eof}, C2};
-        (rsbrace, C2) -> Kv({}, C2);		% empty array
+	(rsbrace, C2) -> Kv({array, []}, C2);  % empty array	     
         (T, C2) -> parse_array([], T, C2, Kv)
     end).
 
