@@ -56,7 +56,7 @@
 -export([parse_set_cookie/1, format_set_cookie/1, 
 	 postvar/2, queryvar/2, getvar/2]).
 
--export([binding/1]).
+-export([binding/1,dir_listing/1, dir_listing/2]).
 
 
 -import(lists, [map/2, flatten/1, reverse/1]).
@@ -1176,13 +1176,19 @@ ehtml_expand(Bin) when binary(Bin) -> Bin; % yaws_api:htmlize(Bin);
 
 ehtml_expand({ssi,File, Del, Bs}) ->
     UT = get(yaws_ut),
-    case yaws_server:ssi(File, Del, Bs, UT#urltype.dir) of
+	ARG = get(yaws_arg),
+
+    case yaws_server:ssi(File, Del, Bs, UT, ARG) of
 	{error, Rsn} ->
 	    io_lib:format("ERROR: ~p~n",[Rsn]);
 	X ->
 	    X
     end;
 
+
+%!todo (low priority) - investigate whether tail-recursion would be of any benefit here instead of the current ehtml_expand(Body) recursion.
+%		- provide a tail_recursive version & add a file in the benchmarks folder to measure it.
+%
 ehtml_expand({Tag}) -> 
     ["<", atom_to_list(Tag), " />"];
 ehtml_expand({pre_html, X}) -> X;
@@ -1309,7 +1315,9 @@ ehtml_expander(Bin, Before, After) when binary(Bin) ->
 
 ehtml_expander({ssi,File, Del, Bs}, Before, After) ->
     UT = get(yaws_ut),
-    Str = case yaws_server:ssi(File, Del, Bs,  UT#urltype.dir) of
+	ARG = get(yaws_arg),
+
+    Str = case yaws_server:ssi(File, Del, Bs, UT, ARG) of
 	      {error, Rsn} ->
 		  io_lib:format("ERROR: ~p~n",[Rsn]);
 	      X ->
@@ -1748,3 +1756,27 @@ setconf(GC0, Groups0) ->
 getconf() ->
     gen_server:call(yaws_server, getconf, infinity).
 
+
+%% Function which invokeable typically from an index.yaws file
+dir_listing(Arg) ->
+    dir_listing(Arg, ".").
+dir_listing(Arg, RelDir) ->
+    %% .yaws.auth
+    Dir0 = filename:dirname(Arg#arg.fullpath),
+    Dir = case RelDir of
+              "." -> Dir0;
+              _ -> filename:join([Dir0, RelDir])
+          end,
+    Req = Arg#arg.req,
+    case file:list_dir(Dir) of
+        {ok, Data0} ->
+            Data = Data0 -- [".yaws.auth", "index.yaws"],
+            yaws_ls:list_directory(Arg, Arg#arg.clisock, Data,
+				   Dir,
+                                   Req,  false),
+	    ok;
+        _Err ->
+            %% Just ignore errors ??, the programmer has to
+            %% make sure it's a valid path here
+	    ok
+    end.
