@@ -9,8 +9,6 @@
 -author('klacke@bluetail.com').
 
 
-
-
 -include("../include/yaws.hrl").
 -include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
@@ -528,9 +526,13 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 	     end;
 
 	["php_exe_path", '=' , PhpPath] ->
-	    fload(FD, globals, GC#gconf{phpexe = PhpPath},
-		  C, Cs, Lno+1, Next);
-
+	    case is_file(PhpPath) of
+		true ->
+		    fload(FD, globals, GC#gconf{phpexe = PhpPath},
+			  C, Cs, Lno+1, Next);
+		false ->
+		    {error, ?F("Expect executable file at line ~w", [Lno])}
+	    end;
 
 	%% deprected, don't use
 	["read_timeout", '=', _Val] ->
@@ -607,7 +609,8 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 	                         GC#gconf.id == default ->
 	    fload(FD, globals, GC#gconf{id=String},C, Cs, Lno+1, Next);
 	["id", '=', String]  ->
-	    error_logger:format("Ignoring 'id = ~p' setting at line ~p~n", [String,Lno]),
+	    error_logger:format("Ignoring 'id = ~p' setting at line ~p~n", 
+				[String,Lno]),
 	    fload(FD, globals, GC, C, Cs, Lno+1, Next);
 	["pick_first_virthost_on_nomatch", '=',  Bool] ->
 	    case is_bool(Bool) of
@@ -627,16 +630,11 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
 		false ->
 		    {error, ?F("Expect true|false at line ~w", [Lno])}
 	    end;
-	["use_large_ssl_pool", '=',  Bool] ->
-	    case is_bool(Bool) of
-		{true, Val} ->
-		    fload(FD, globals, 
-			  ?gc_set_use_large_ssl_pool(GC,Val),
-			  C, Cs, Lno+1, Next);
-		false ->
-		    {error, ?F("Expect true|false at line ~w", [Lno])}
-	    end;
-	
+	["use_large_ssl_pool", '=',  _Bool] ->
+	    error_logger:format("ssl_pool is deprecated and not used any more\n",
+				[]),
+	    fload(FD, globals, GC,
+		  C, Cs, Lno+1, Next);
 
 	['<', "server", Server, '>'] ->  %% first server 
             fload(FD, server, GC, #sconf{servername = Server},
@@ -740,7 +738,7 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 		    C2 = C#sconf{docroot = hd(RootDirs),
 				 xtra_docroots = tl(RootDirs)},
 		    fload(FD, server, GC, C2, Cs, Lno+1, Next);
-		false ->
+		_ ->
 		    {error, ?F("Expect directory at line ~w", [Lno])}
 	    end;
 
@@ -755,7 +753,9 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 			    C2 = C#sconf{partial_post_size = I},
 			    fload(FD, server, GC, C2, Cs, Lno+1, Next);
 			_ ->
-			    {error, ?F("Expect integer or 'nolimit' at line ~w", [Lno])}
+			    {error, 
+			     ?F("Expect integer or 'nolimit' at line ~w", 
+				[Lno])}
 		    end
 	    end;
 	['<', "auth", '>'] ->
@@ -821,12 +821,6 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
 				   Suffixes)},
 	    fload(FD, server, GC, C2, Cs, Lno+1, Next);
 
-	["php_exe_path", '=' , _PhpPath] ->
-	    error_logger:format("php_exe_path must now be specified in"
-				" in the global part of the conf instead of"
-				"within the server ",[]),
-	    fload(FD, server, GC, C, Cs, Lno+1, Next);
-
 	["revproxy", '=', Prefix, Url] ->
 	    case (catch yaws_api:parse_url(Url)) of
 		{'EXIT', _} ->
