@@ -78,18 +78,6 @@ actl_trace(What) ->
 
 
 
-%% change owner of logfiles and logdir
-%% only called when we lower rights after socket(s) are opened
-% uid_change(GC) ->
-%     case file:list_dir(GC#gconf.logdir) of
-% 	{ok, L} ->
-% 	    yaws:uid_change_files(GC, GC#gconf.logdir,L);
-% 	{error, Rsn} ->
-% 	    error_logger:format("Failed to listdir ~p : ~p",
-% 				[GC#gconf.logdir, Rsn])
-%     end.
-
-
 
 %%%----------------------------------------------------------------------
 %%% Callback functions from gen_server
@@ -334,7 +322,11 @@ do_alog(ServerName, Ip, User, Req, Status, Length, Referrer,
 
 do_auth_log(ServerName, IP, Path, Item, State) ->
     AL = State#state.auth_log,
-    I = [inet_parse:ntoa(IP), 
+    IpStr = case catch inet_parse:ntoa(IP) of
+		{'EXIT', _} -> "unknownip";
+		Val -> Val
+	    end,
+    I = [IpStr,
 	 " ", State#state.now,
 	 " ", ServerName, " " ,"\"", Path,"\"",
 	 case Item of
